// Dart imports:
import 'dart:async';

// Package imports:
import 'package:zego_express_engine/zego_express_engine.dart';

// Project imports:
import 'package:zego_uikit/src/services/services.dart';

mixin ZegoUIKitCoreDataError {
  StreamController<ZegoUIKitError>? errorStreamCtrl;

  void initError() {
    ZegoLoggerService.logInfo(
      'init error',
      subTag: 'core data',
    );

    errorStreamCtrl ??= StreamController<ZegoUIKitError>.broadcast();
  }

  void uninitError() {
    ZegoLoggerService.logInfo(
      'uninit error',
      subTag: 'core data',
    );

    errorStreamCtrl?.close();
    errorStreamCtrl = null;
  }

  void onDebugError(int errorCode, String funcName, String info) {
    errorStreamCtrl?.add(
      ZegoUIKitError(
        code: errorCode,
        message: info,
        method: 'express-debug:$funcName',
      ),
    );
  }

  void onFatalError(int errorCode) {
    errorStreamCtrl?.add(
      ZegoUIKitError(
        code: errorCode,
        message: '',
        method: 'express-fatal',
      ),
    );
  }

  void onApiCalledResult(int errorCode, String funcName, String info) {
    if (ZegoErrorCode.CommonSuccess != errorCode) {
      errorStreamCtrl?.add(
        ZegoUIKitError(
          code: errorCode,
          message: info,
          method: 'express-api:$funcName',
        ),
      );
    }
  }

  void onMobileScreenCaptureExceptionOccurred(
    ZegoScreenCaptureExceptionType exceptionType,
  ) {
    errorStreamCtrl?.add(
      ZegoUIKitError(
        code: ZegoUIKitErrorCode.fromZegoScreenCaptureExceptionType(
          exceptionType,
        ),
        message: exceptionType.name,
        method: 'express-onMobileScreenCaptureExceptionOccurred',
      ),
    );
  }
}
